Package: sessioninfo
Title: R Session Information
Version: 1.2.4
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "cre"),
    person("Hadley", "Wickham", role = "aut"),
    person("Winston", "Chang", role = "aut"),
    person("Robert", "Flight", role = "aut"),
    person("Kirill", "Müller", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("R Core team", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Description: Query and print information about the current R session.  It
    is similar to 'utils::sessionInfo()', but includes more information
    about packages, and where they were installed from.
License: GPL-2
URL: https://sessioninfo.r-lib.org,
        https://github.com/r-lib/sessioninfo
BugReports: https://github.com/r-lib/sessioninfo/issues
Depends: R (>= 3.4)
Imports: cli (>= 3.1.0), tools, utils
Suggests: callr, covr, gh, reticulate, rmarkdown, testthat (>= 3.2.0),
        withr
Config/Needs/website: pkgdown, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/usethis/last-upkeep: 2025-05-07
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-04 13:12:01 UTC; gaborcsardi
Author: Gábor Csárdi [cre],
  Hadley Wickham [aut],
  Winston Chang [aut],
  Robert Flight [aut],
  Kirill Müller [aut],
  Jim Hester [aut],
  R Core team [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Repository: CRAN
Date/Publication: 2026-06-04 13:50:02 UTC
Built: R 4.6.0; ; 2026-06-04 14:24:20 UTC; unix
