# Generated by roxygen2: do not edit by hand

S3method(as.character,external_info)
S3method(as.character,packages_info)
S3method(as.character,platform_info)
S3method(as.character,python_info)
S3method(as.character,session_info)
S3method(c,platform_info)
S3method(format,external_info)
S3method(format,packages_info)
S3method(format,platform_info)
S3method(format,python_info)
S3method(format,session_diff)
S3method(format,session_info)
S3method(print,external_info)
S3method(print,packages_info)
S3method(print,platform_info)
S3method(print,python_info)
S3method(print,session_diff)
S3method(print,session_info)
export(external_info)
export(os_name)
export(package_info)
export(platform_info)
export(python_info)
export(session_diff)
export(session_info)
importFrom(cli,symbol)
importFrom(utils,packageVersion)
